// list.c
//
//  list/display functions
//
//  November, 1998
//  jerry@mail.csh.rit.edu
#include <stdio.h>
#include <string.h>
#include "allegro.h"
#include "general.h"
#include "guipal.h"
#include "sprtplte.h"
#include "util.h"
#include "gamedesc.h"
#include "inidriv.h"
#include "button.h"


// structure to store all of the file info into...
typedef struct f_st {
    char filename[127];
    char gamename_short[32];
    char gamename_long[127];
} FILE_ST;


FILE_ST * fst = NULL;
int fst_total;
int fst_count;

#define FST_COUNT (0)
#define FST_BASIC (1)
#define FST_FULL  (2)
#define FST_ROMS  (3)

void fst_callback(char * fname, int attrib, int param)
{
    char * pBuffer = NULL;
    char ** argv;
    char section[16];
    int argc;
    int count;

    if (param != FST_COUNT)
    {
        // then the fst pointer is set up, otherwise, it's just counting

        sprintf(fst[fst_count].filename, fname);

	pBuffer = get_filename(fname);
	strncpy(fst[fst_count].gamename_short, pBuffer, 32);

	// strip off the ".ini" extension...
	for(count=0 ; count < 12; count++)
	    if (fst[fst_count].gamename_short[count] == '.')
		fst[fst_count].gamename_short[count] = '\0';

	if (param == FST_FULL || param == FST_ROMS)
	{
	    push_config_state();
	    set_config_file(fname);

	    pBuffer = get_config_string("General", "Description", "None");
	    if (strcmp(pBuffer, "None"))
	    {
		strncpy(fst[fst_count].gamename_long, pBuffer, 126);
		GameDescription[126] = 0;
	    } else {
		sprintf(fst[fst_count].gamename_long, "Badly formatted file.");
		sprintf(fst[fst_count].gamename_short, "???");
	    }

	    if (param == FST_ROMS)
	    {
		// we'll just print this all out from in here.
		// it reduces the chance of a memory munging, and prints
		// out as it goes, rather than waiting till it's done.

		printf("%-8s \"%s\"\n", 
			 fst[fst_count].gamename_short, 
			 fst[fst_count].gamename_long);

		count = 1;
		for(;;)
		{
		    sprintf(section, "Rom%d", count);
		    argv = get_config_argv("GraphicsRoms", section, &argc);
		    if (argc <3) break;

		    printf("    %s\n", argv[2]);
		    count++;
		}
		printf("\n");
	    }

	    pop_config_state();

	}
        fst_count++;
    }
}


// for the qsort function...
int fst_cmp(const void *e1, const void *e2)
{
    FILE_ST *fst1 = (FILE_ST *)e1;
    FILE_ST *fst2 = (FILE_ST *)e2;

    if (e1 == NULL || e2 == NULL)  return 0;

    return ( strcmp(fst1->gamename_short, fst2->gamename_short) );
}

void create_fst_list(int task)
{
    // construct the file list...
    fst_total = for_each_file("drivers\\*.ini", 0, fst_callback, FST_COUNT);

    if (fst)  free(fst);
    fst = (FILE_ST *) malloc(sizeof(FILE_ST) * fst_total);
    if (fst)
    {
	fst_count = 0;
	(void) for_each_file("drivers\\*.ini", 0, fst_callback, task);

	qsort(fst, fst_total, sizeof(FILE_ST), fst_cmp); // in stdlib  :D
    }
}

void destroy_fst_list(void)
{
    if(fst) free(fst);
}

////////////////////////////////////////////////////////////////////////////////


void list_games(void)
{
    int count;
    int online;

    create_fst_list(FST_BASIC);

    printf("\nTURACO currently supports the following games:\n");
    online=99;
    for (count = 0 ; count < fst_total ; count++)
    {
        if (online >= 7)
        {
	    printf("\n");
	    online = 0;
        }
	printf("  %-8s", fst[count].gamename_short);
	online++;
    }
    printf("\n\nTotal ROM sets supported:  %d\n\n", fst_total);

    destroy_fst_list();
}


void list_html(void)
{
    int count;
    int online;

    create_fst_list(FST_BASIC);

    printf("<!-- BEGIN TURACO LIST>\n");
    printf("<!table generated by \"TURACO.EXE -listhtml\">\n\n");
    printf("<table border = 0>\n\n");
    printf("<tr>\n");
    printf("<td colspan=7 bgcolor=\"444444\">\n"
           "<center>\n"
	   "TURACO currently supports the following games:"
	   "</center>\n"
	   "</td>\n");

    online=99;
    for (count = 0 ; count < fst_total ; count++)
    {
        if (online >= 7)
        {
	    printf("</tr>\n\n<tr>\n");
	    online = 0;
        }
	printf("<td bgcolor=\"333333\"> <center> &nbsp "
	       "%s"
	       " &nbsp </center> </td>\n", fst[count].gamename_short);
	online++;
    }
    printf("</tr>\n\n<tr>\n");
    printf("<td colspan=7 bgcolor=\"444444\">\n");
    printf("<center>Total ROM sets supported:  %d"
           "</center>\n", fst_total);
    printf("</td>\n</tr>\n</table>\n\n");
    printf("<!-- END TURACO LIST>\n");

    destroy_fst_list();
}


void list_full(void)
{
    int count;

    create_fst_list(FST_FULL);

    printf("\nTURACO currently supports the following games:\n");
    for (count = 0 ; count < fst_total ; count++)
	printf(" %-8s    \"%s\"\n", 
	         fst[count].gamename_short, 
	         fst[count].gamename_long);

    printf("\n");

    destroy_fst_list();
}

void list_roms(void)
{
    printf("\nTURACO currently supports the following games:\n\n");

    // we'll display the bulk of this in the callback functions.
    // it won't be perfectly alphebetized. but it should be good enough

    // therefore, we don't need to do more there than just create and
    // destroy the list...
    create_fst_list(FST_ROMS);
    destroy_fst_list();

    printf("\n");
}
